@@ -71,6 +71,8 @@ item(MULTIPLE)
 item(TIMESTAMP)
 item(VERSION)
 item(ATOM_PAIR)
+item(_XROOTPMAP_ID)
+item(_XSETROOT_ID)
 
 /* Oddities: These are used, and we need atoms for them,
  * but when we need all _NET_WM hints (i.e. when we're making
@@ -107,7 +109,12 @@ item(_NET_WM_WINDOW_TYPE_DESKTOP)
 item(_NET_WM_WINDOW_TYPE_DOCK)
 item(_NET_WM_WINDOW_TYPE_TOOLBAR)
 item(_NET_WM_WINDOW_TYPE_MENU)
+item(_NET_WM_WINDOW_TYPE_UTILITY)
+item(_NET_WM_WINDOW_TYPE_SPLASH)
 item(_NET_WM_WINDOW_TYPE_DIALOG)
+item(_NET_WM_WINDOW_TYPE_DROPDOWN_MENU)
+item(_NET_WM_WINDOW_TYPE_TOOLTIP)
+item(_NET_WM_WINDOW_TYPE_DND)
 item(_NET_WM_WINDOW_TYPE_NORMAL)
 item(_NET_WM_STATE_MODAL)
 item(_NET_CLIENT_LIST)
@@ -121,8 +128,6 @@ item(_NET_WM_MOVERESIZE)
 item(_NET_ACTIVE_WINDOW)
 item(_NET_WM_STRUT)
 item(_NET_WM_STATE_HIDDEN)
-item(_NET_WM_WINDOW_TYPE_UTILITY)
-item(_NET_WM_WINDOW_TYPE_SPLASH)
 item(_NET_WM_STATE_FULLSCREEN)
 item(_NET_WM_PING)
 item(_NET_WM_PID)
@@ -159,6 +164,7 @@ item(_NET_WM_STATE_STICKY)
 item(_NET_WM_FULLSCREEN_MONITORS)
 item(_NET_WM_STATE_FOCUSED)
 item(_NET_RESTACK_WINDOW)
+item(_NET_WM_WINDOW_OPACITY)
 
 /* eof atomnames.h */
 
