@@ -24,7 +24,8 @@
  * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <config.h>
+#include "config.h"
+
 #include "screen-private.h"
 #include "util.h"
 #include "errors.h"
@@ -35,7 +36,7 @@
 #include "keybindings.h"
 #include "stack.h"
 #include "xprops.h"
-#include "compositor.h"
+#include "meta-compositor.h"
 
 #ifdef HAVE_SOLARIS_XINERAMA
 #include <X11/extensions/xinerama.h>
@@ -635,11 +636,7 @@ meta_screen_free (MetaScreen *screen,
 
   meta_display_grab (display);
 
-  if (screen->display->compositor)
-    {
-      meta_compositor_unmanage_screen (screen->display->compositor,
-				       screen);
-    }
+  meta_compositor_unmanage_screen (display->compositor, screen);
 
   meta_display_unmanage_windows_for_screen (display, screen, timestamp);
 
@@ -786,9 +783,8 @@ meta_screen_manage_all_windows (MetaScreen *screen)
         continue;
       }
 
-      if (screen->display->compositor)
-        meta_compositor_add_window (screen->display->compositor, window,
-                                    info->xwindow, &info->attrs);
+      meta_compositor_add_window (screen->display->compositor, window,
+                                  info->xwindow, &info->attrs);
     }
   meta_stack_thaw (screen->stack);
 
@@ -805,8 +801,6 @@ meta_screen_composite_all_windows (MetaScreen *screen)
   GList *windows, *list;
 
   display = screen->display;
-  if (!display->compositor)
-    return;
 
   windows = list_windows (screen);
 
@@ -815,6 +809,7 @@ meta_screen_composite_all_windows (MetaScreen *screen)
   for (list = windows; list != NULL; list = list->next)
     {
       WindowInfo *info = list->data;
+      MetaWindow *window;
 
       if (info->xwindow == screen->no_focus_window ||
           info->xwindow == screen->flash_window ||
@@ -824,10 +819,9 @@ meta_screen_composite_all_windows (MetaScreen *screen)
         continue;
       }
 
-      meta_compositor_add_window (display->compositor,
-                                  meta_display_lookup_x_window (display,
-                                                                info->xwindow),
-				  info->xwindow, &info->attrs);
+      window = meta_display_lookup_x_window (display, info->xwindow);
+      meta_compositor_add_window (display->compositor, window,
+                                  info->xwindow, &info->attrs);
     }
 
   meta_stack_thaw (screen->stack);
@@ -843,10 +837,10 @@ meta_screen_for_x_screen (Screen *xscreen)
 
   display = meta_display_for_x_display (DisplayOfScreen (xscreen));
 
-  if (display == NULL)
+  if (display == NULL || display->screen->xscreen != xscreen)
     return NULL;
 
-  return meta_display_screen_for_x_screen (display, xscreen);
+  return display->screen;
 }
 
 static void
@@ -1213,12 +1207,13 @@ get_window_pixbuf (MetaWindow *window,
                    int        *width,
                    int        *height)
 {
+  MetaDisplay *display;
   cairo_surface_t *surface;
   GdkPixbuf *pixbuf, *scaled;
   double ratio;
 
-  surface = meta_compositor_get_window_surface (window->display->compositor,
-                                            window);
+  display = window->display;
+  surface = meta_compositor_get_window_surface (display->compositor, window);
   if (surface == NULL)
     return NULL;
 
@@ -1456,13 +1451,12 @@ meta_screen_tile_preview_update_timeout (gpointer data)
 {
   MetaScreen *screen = data;
   MetaWindow *window = screen->display->grab_window;
-  gboolean composited = screen->display->compositor != NULL;
   gboolean needs_preview = FALSE;
 
   screen->tile_preview_timeout_id = 0;
 
   if (!screen->tile_preview)
-    screen->tile_preview = meta_tile_preview_new (composited);
+    screen->tile_preview = meta_tile_preview_new ();
 
   if (window)
     {
