@@ -49,8 +49,6 @@ void meta_ui_init (int *argc, char ***argv);
 
 Display* meta_ui_get_display (void);
 
-gint     meta_ui_get_screen_number (void);
-
 void meta_ui_add_event_func    (Display       *xdisplay,
                                 MetaEventFunc  func,
                                 gpointer       data);
@@ -132,10 +130,8 @@ MetaWindowMenu* meta_ui_window_menu_new   (MetaUI             *ui,
                                            MetaWindowMenuFunc  func,
                                            gpointer            data);
 void            meta_ui_window_menu_popup (MetaWindowMenu     *menu,
-                                           int                 root_x,
-                                           int                 root_y,
-                                           int                 button,
-                                           guint32             timestamp);
+                                           const GdkRectangle *rect,
+                                           const GdkEvent     *event);
 void            meta_ui_window_menu_free  (MetaWindowMenu     *menu);
 
 
