@@ -1655,9 +1655,17 @@ meta_frame_titlebar_event (MetaFrames     *frames,
       break;
 
     case G_DESKTOP_TITLEBAR_ACTION_MENU:
-      meta_core_show_window_menu (frames->xdisplay, frame->xwindow,
-                                  event->x_root, event->y_root,
-                                  event->button, event->time);
+      {
+        GdkRectangle rect;
+
+        rect.x = event->x;
+        rect.y = event->y;
+        rect.width = 0;
+        rect.height = 0;
+
+        meta_core_show_window_menu (frames->xdisplay, frame->xwindow,
+                                    &rect, event);
+      }
       break;
 
     default:
@@ -1812,7 +1820,6 @@ meta_frames_button_press_event (GtkWidget      *widget,
         {
           MetaFrameGeometry fgeom;
           GdkRectangle rect;
-          int dx, dy;
 
           meta_frames_calc_geometry (frames, frame, &fgeom);
 
@@ -1822,21 +1829,10 @@ meta_frames_button_press_event (GtkWidget      *widget,
               return FALSE;
             }
 
-          /* get delta to convert to root coords */
-          dx = event->x_root - event->x;
-          dy = event->y_root - event->y;
-
-          /* Align to the right end of the menu rectangle if RTL */
-          if (meta_ui_get_direction() == META_UI_DIRECTION_RTL)
-            dx += rect.width;
-
           frame->ignore_leave_notify = TRUE;
           meta_core_show_window_menu (frames->xdisplay,
                                       frame->xwindow,
-                                      rect.x + dx,
-                                      rect.y + rect.height + dy,
-                                      event->button,
-                                      event->time);
+                                      &rect, event);
         }
     }
   else if (event->button == 1 &&
@@ -2229,31 +2225,6 @@ meta_frames_motion_notify_event     (GtkWidget           *widget,
   return TRUE;
 }
 
-static void
-setup_bg_cr (cairo_t *cr, GdkWindow *window, int x_offset, int y_offset)
-{
-  GdkWindow *parent = gdk_window_get_parent (window);
-  cairo_pattern_t *bg_pattern;
-
-  G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-  bg_pattern = gdk_window_get_background_pattern (window);
-  G_GNUC_END_IGNORE_DEPRECATIONS
-
-  if (bg_pattern == NULL && parent)
-    {
-      gint window_x, window_y;
-
-      gdk_window_get_position (window, &window_x, &window_y);
-      setup_bg_cr (cr, parent, x_offset + window_x, y_offset + window_y);
-    }
-  else if (bg_pattern)
-    {
-      cairo_translate (cr, - x_offset, - y_offset);
-      cairo_set_source (cr, bg_pattern);
-      cairo_translate (cr, x_offset, y_offset);
-    }
-}
-
 /* Returns a pixmap with a piece of the windows frame painted on it.
 */
 
@@ -2276,7 +2247,6 @@ generate_pixmap (MetaFrames            *frames,
   cr = cairo_create (result);
   cairo_translate (cr, -rect->x, -rect->y);
 
-  setup_bg_cr (cr, frame->window, 0, 0);
   cairo_paint (cr);
 
   meta_frames_paint (frames, frame, cr);
